///	\file	menu.h
///	\brief	menu.h
#ifndef Menu_h_
#define Menu_h_

#include <string>
#include <vector>
#include <ncurses.h>

namespace MLS {

class Panel;

///	\brief	MenuItem class
class MenuItem
{
	std::string name;		///< 화면에 출력될 MenuItem 이름
	std::string command;	///< 실행될 MenuItem 명령어
	
public:
	
	///	\brief	생성자
	///	\param	n		MenuItem 이름
	///	\param	c		MenuItem 명령어
	MenuItem(const std::string &n, const std::string &c) 
		: name(n), command(c) 
	{}
	
	// setter	===================================
	///	\brief	이름 설정함수
	///	\param	n		설정할 화면에 출력될 메뉴 Item
	void SetName(const std::string &n) {name = n;}

	///	\brief	명령 설정 함수
	///	\param	n		  설정할 명령어
	void SetCmd(const std::string &n) {command = n;}	
	// ============================================
	
	// getter	===================================
	///	\brief	화면에 출력될 이름 얻기 함수
	///	\return	화면에 출력될 이름
	const std::string &GetName() {return name;}

	///	\brief	명령어 얻기 함수
	///	\return	명령어
	const std::string &GetCmd() {return command;}		
	// ============================================
};

// 메뉴의 분류
///	\brief	MenuCategory class[메뉴의 분류]
class MenuCategory
{
	std::string name;		///< 메뉴 이름
	
	int index;     			///< 메뉴 Category 내에서 현재 커서 위치
	int location;			///< 상위 메뉴 내에서의 Category 위치
	
	std::vector<MenuItem> item; ///< 서브메뉴
	
	WINDOW *win;			///< Main WINDOW(ncurses)
	
public:
	///	\brief	생성자
	///	\param	p		MenuCategory 이름
	///	\param	l		MenuCategory location
	MenuCategory(const std::string &p, const int &l = 0)
		: name(p) , index(0), location(l), win(NULL)
	{}

	///	\brief	소멸자
	~MenuCategory()
	{
		if (win) delwin(win);
	}
			
	///	\brief	이름  얻기 함수
	///	\return	이름
	const std::string &GetName(){return name;}
	
	///	\brief	MenuItem 추가 함수
	///	\param	n		MenuItem 이름
	///	\param	c		MenuItem Command
	void AddItem(const std::string &n, const std::string &c)
	{
		item.push_back(MenuItem(n,c));
	}
	
	///	\brief	위치 설정 함수
	///	\param	x	설정될 위치
	void SetLocation(const int &x){ location = x;}

	///	\brief	위치 얻기 함수
	///	\return	위치
	int  GetLocation() { return location;}
	
	///	\brief	MenuItem Command 얻기함수
	///	\return	MenuItem Command
	const std::string &GetCmd() { return item[index].GetCmd(); }

	void Draw();
	void Create();
	
	void CursorUp();
	void CursorDown();	
};

///	\brief	Menu class
class Menu
{
protected:
	int num;						///< 현재 선택되어 있는 item index
	std::vector<MenuCategory> item; ///< 메뉴 뭉치들(예.윈도우에서 File 메뉴)
	
	WINDOW *win;					///< nCurses WINDOW
	
public:
	Menu();
	~Menu();
	
	///	\brief	menuCategory를 추가한다.
	///	\param	p	추가할 MenuCategory
	void AddItem(MenuCategory p)
	{
		// . 아무런 메뉴(예. File 메뉴)가 없으면 초기위치(x)를 4로한다.
		if (item.empty()) p.SetLocation(4);
		// . 메뉴가 있다면 전 메뉴가 위치한 자리와 글자수를 참고하여 위치를 설정
		else p.SetLocation(item.back().GetLocation() + item.back().GetName().size() + 2);

		// . 메뉴 뭉치 추가
		item.push_back(p);
	}
	
	///	\brief	menuCategory를 추가한다.
	///	\param	p	추가할 메뉴 문자열
	void AddItem(const std::string &p)
	{
		AddItem(MenuCategory(p));
	}
	
	void Create();
	void CreateWindow();
	
	void Draw();	
	void Process(MLS::Panel &);

};

}; // namespace

#endif 
